package swingtables.gui;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.table.TableRowSorter;
import swingtables.logic.entities.DbGenre;
import swingtables.logic.MoviesDatasource;

public class SwingTablesFrame extends JFrame {

    public static void main(String[] args) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new SwingTablesFrame().setVisible(true);
            }
        });
    }

    static void displayError(Exception ex) {
        ex.printStackTrace(System.err);
        JOptionPane.showMessageDialog(null, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
    }
    private JTable moviesTable;
    private MoviesTableModel moviesTableModel;
    private TableRowSorter<MoviesTableModel> moviesTableRowSorter;

    public SwingTablesFrame() throws HeadlessException {
        setTitle("javax.swing.JTable");
        setLocation(30, 30);
        setSize(800, 600);
        setLayout(new BorderLayout());
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        moviesTableModel = new MoviesTableModel();
        moviesTableRowSorter = new TableRowSorter<>(moviesTableModel);
        moviesTable = new JTable(moviesTableModel);
        getContentPane().add(new JScrollPane(moviesTable));
        moviesTable.setRowSorter(moviesTableRowSorter);
        moviesTable.getColumn("Genre").setCellEditor(new DefaultCellEditor(new JComboBox<>(MoviesDatasource.getInstance().getGenres().toArray(new DbGenre[0]))));
        moviesTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        setJMenuBar(new TablesMenu());
    }

    private class TablesMenu extends JMenuBar {

        private final Action newTitleFilterAction = new AbstractAction("New filter for titles") {
            @Override
            public void actionPerformed(ActionEvent e) {
                String filter = JOptionPane.showInputDialog("Please, specify a filter for the movie titles");
                RowFilter<MoviesTableModel, Integer> rf = RowFilter.regexFilter(filter, 0);
                moviesTableRowSorter.setRowFilter(rf);

            }
        };
        private final Action newGenreFilterAction = new AbstractAction("New filter for genres") {
            @Override
            public void actionPerformed(ActionEvent e) {
                List<DbGenre> genres;
                genres = MoviesDatasource.getInstance().getGenres();
                if (genres.isEmpty()) {
                    JOptionPane.showMessageDialog(rootPane, "There are no genres");
                    return;
                }
                final DbGenre genreFilter = (DbGenre) JOptionPane.showInputDialog(rootPane, "Please, select a genre for filtering", "", JOptionPane.QUESTION_MESSAGE, null, genres.toArray(new DbGenre[0]), genres.get(0));
                if (genreFilter == null) {
                    return;
                }
                RowFilter<MoviesTableModel, Integer> rf = new RowFilter<MoviesTableModel, Integer>() {
                    @Override
                    public boolean include(RowFilter.Entry entry) {
                        return entry.getValue(3).equals(genreFilter);
                    }
                };
                moviesTableRowSorter.setRowFilter(rf);
            }
        };
        private final Action clearFilterAction = new AbstractAction("Clear filter") {
            @Override
            public void actionPerformed(ActionEvent e) {
                moviesTableRowSorter.setRowFilter(null);
            }
        };
        private final Action addMovieAction = new AbstractAction("Add new movie") {
            @Override
            public void actionPerformed(ActionEvent e) {
                moviesTableModel.addNewMovie();
            }
        };
        private final Action deleteMovieAction = new AbstractAction("Delete selected movies") {
            @Override
            public void actionPerformed(ActionEvent e) {
                int[] selectedRows = moviesTable.getSelectedRows();
                for (int selectedRow : selectedRows) {
                    int rowIndex = moviesTableRowSorter.convertRowIndexToModel(selectedRow);
                    moviesTableModel.deleteMovie(rowIndex);
                }
            }
        };

        public TablesMenu() {
            JMenu jMenu = new JMenu("Menu");
            add(jMenu);
            jMenu.add(newTitleFilterAction);
            jMenu.add(newGenreFilterAction);
            jMenu.add(clearFilterAction);
            jMenu.add(new JSeparator());
            jMenu.add(addMovieAction);
            jMenu.add(deleteMovieAction);
        }
    }
}
