package swingtables.logic;

import swingtables.logic.entities.DbMovie;
import swingtables.logic.entities.DbGenre;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import swingtables.logic.entities.DbGenreJpaController;
import swingtables.logic.entities.DbMovieJpaController;
import swingtables.logic.entities.exceptions.NonexistentEntityException;

public class MoviesDatasource {

    private static MoviesDatasource moviesDatasourceInstance = null;

    public static MoviesDatasource getInstance() {
        if (moviesDatasourceInstance == null) {
            moviesDatasourceInstance = new MoviesDatasource();
        }
        return moviesDatasourceInstance;
    }
    private DbGenreJpaController genreJpaController;
    private DbMovieJpaController movieJpaController;

    private MoviesDatasource() {
        genreJpaController = new DbGenreJpaController(getEmf());
        movieJpaController = new DbMovieJpaController(getEmf());
    }

    final EntityManagerFactory getEmf() {
        return Persistence.createEntityManagerFactory("SimpleJpaPreziPU");
    }

    public List<DbMovie> getMovies() {
        List<DbMovie> movies = movieJpaController.findDbMovieEntities();
        return movies;
    }

    public List<DbGenre> getGenres() {
        List<DbGenre> genres = genreJpaController.findDbGenreEntities();
        return genres;
    }

    public int addMovie() {
        DbMovie dbMovie = new DbMovie();
        dbMovie.setDefaultValues();
        movieJpaController.create(dbMovie);
        return getMoviesCount() - 1;
    }

    public void deleteMovie(int index) throws NonexistentEntityException {
        DbMovie movie = movieJpaController.findDbMovieEntities().get(index);
        movieJpaController.destroy(movie.getId());
    }

    public void updateMovie(DbMovie movie) throws NonexistentEntityException, Exception {
        movieJpaController.edit(movie);
    }

    public int getMoviesCount() {
        return movieJpaController.getDbMovieCount();
    }

    public DbMovie getMovie(int rowIndex) {
        return movieJpaController.findDbMovieEntities(1, rowIndex).get(0);
    }
}
